package com.perforce.p4simulink.connection;

/**
 * Copyright (C) 2014 Perforce Software. All rights reserved.
 *
 * Please see LICENSE.txt in top-level folder of this distribution.
 */

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import com.perforce.p4simulink.P4CMException;

public class Config {
	private static final Logger log = LogManager.getLogger(Config.class
			.getName());

	public static enum AuthType {
		PASSWORD, TICKETPATH, NONE
	}

	private File file = null;
	private Properties properties = new Properties();
	private boolean loaded = false;
	private AuthType authType = AuthType.PASSWORD;

	public Config() {
		this(null);
	}

	public Config(File dir) {
		if (dir != null && dir.isDirectory()) {
			file = new File(dir.getAbsolutePath() + File.separator
					+ ".p4config");
		} else {
			file = dir;
		}
	}

	public File getConfigFile() {
		return file;
	}

	public Config setLoaded(boolean loaded) {
		this.loaded = loaded;
		return this;
	}

	public boolean exists() {
		return file != null && file.isFile() && file.canRead();
	}

	public void load() throws P4CMException {
		load(false);
	}

	/**
	 * Load the config file into memory, making sure that at least P4PORT,
	 * P4USER and P4CLIENT are set.
	 *
	 * @throws com.perforce.p4simulink.P4CMException
	 */
	public void load(boolean force) throws P4CMException {
		// if we're already loaded and we're not being forced, we're done
		if (loaded && !force) {
			return;
		}

		if (file.exists() && file.isFile()) {
			try (FileInputStream in = new FileInputStream(file)) {
				load(in);
				loaded = true;
			} catch (IOException e) {
				log.error("Config.load() - failed: " + e.getMessage());
			}
		}
	}

	/**
	 * Saves the current configuration values to the config files. Note that we
	 * only ever save user, port and client (workspace). We do not save
	 * passwords, ticket values or ticket filenames.
	 *
	 * @throws P4CMException
	 */
	public void save() throws P4CMException {
		try (BufferedWriter out = new BufferedWriter(new FileWriter(file))) {
			String output = "";
			output += appendVariable("P4USER", getUser());
			output += appendVariable("P4PORT", getPort());
			output += appendVariable("P4CLIENT", getClient());
			out.write(output);
			out.close();
		} catch (IOException e) {
			log.error("Error saving config: " + e.getLocalizedMessage());
			throw new P4CMException(e);
		}
	}

	private String appendVariable(String name, String value) {
		if (value == null || value.length() == 0) {
			return "";
		}

		return name + "=" + value + "\n";
	}

	/**
	 * Loads Perforce config information from the specified input stream.
	 *
	 * @param is
	 *            InputStream to load from
	 * @throws P4CMException
	 */
	public void load(InputStream is) throws P4CMException {
		properties = new Properties();
		try {
			properties.load(is);

			// check for P4TICKETS environment variable
			String p4tickets = System.getenv("P4TICKETS");
			if (p4tickets != null && !p4tickets.isEmpty()) {
				properties.setProperty("P4TICKETS", p4tickets);
			}

			if (properties.containsKey("P4TICKETS")) {
				// check for file
				File ticketFile = new File(properties.getProperty("P4TICKETS"));
				if (!ticketFile.isDirectory() && !ticketFile.canRead()) {
					authType = AuthType.TICKETPATH;
				}
			}

			log.debug("Perforce configuration loaded: {}",
					properties.toString());
		} catch (IOException e) {
			// we might need to recover from this later
			log.error("Config.load() - failed: " + e.getMessage());
		}
	}

	/**
	 * Extracts P4PORT from the loaded config and returns a P4Java suitable URI.
	 *
	 * @return P4PORT converted to a P4Java URI
	 */
	public String getP4JavaURI() {
		String port = getProperty("P4PORT");

		if (port == null || port.isEmpty()) {
			return "";
		}

		// strip any protocols
		port = port.replaceAll(".+://", "");

		String[] parts = port.split(":", 2);
		if (parts.length != 2) {
			return "";
		}

		StringBuilder address = new StringBuilder();
		if (parts[0].startsWith("ssl")) {
			address.append("p4javassl://");
			address.append(parts[1]);
		} else {
			address.append("p4java://");
			address.append(port);
		}
		log.debug("Perforce Server: {}", address.toString());
		return address.toString();
	}

	public String getPort() {
		return getProperty("P4PORT");
	}

	public Config setPort(String address) {
		// ensure we strip any protocols off the address
		address = address.replaceAll(".+://", "");
		properties.setProperty("P4PORT", address);
		return this;
	}

	public String getClient() {

		return getProperty("P4CLIENT");
	}

	public Config setClient(String client) {
		properties.setProperty("P4CLIENT", client);
		return this;
	}

	public String getUser() {
		return getProperty("P4USER");
	}

	public Config setUser(String user) {
		properties.setProperty("P4USER", user);
		return this;
	}

	public AuthType getAuthType() {
		return authType;
	}

	public String getTicketPath() {
		return getProperty("P4TICKETS");
	}

	public String getPassword() {
		return getProperty("P4PASSWD");
	}

	public Config setPassword(String password) {
		properties.setProperty("P4PASSWD", password);
		authType = AuthType.PASSWORD;
		return this;
	}

	@Override
	public String toString() {
		String port = getProperty("P4PORT");
		String user = getProperty("P4USER");
		String client = getProperty("P4CLIENT");
		String tickets = getProperty("P4TICKETS");
		String configFile = this.file != null ? this.file.getAbsolutePath()
				: "";
		return "CLIENT: '" + client + "' USER: '" + user + "' PORT: '" + port
				+ "' TICKETS: '" + tickets + "' CONFIG: " + configFile;
	}

	private String getProperty(String key) {
		if (properties != null && properties.containsKey(key)) {
			return properties.getProperty(key);
		}
		return "";
	}
}
